#!/usr/bin/python
"""
MemInfo - memory info utility

Copyright (C) 2001 James ''Wez''Weatherall

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""

try: import pylau
except ImportError: pass

import gtk
import sys

def gdk_intern_atom(name):
    import _gtk
    return _gtk.gdk_atom_intern(name, 1)

def set_gdk_win_prop(window, property, type,
                     format, mode, data):
    import _gtk, GDK
    atom = _gtk.gdk_atom_intern(property, 1)
    if not int(atom): return 0
    int_atom = _gtk.gdk_atom_intern(type, 1)
    if not int(int_atom): return 0
    if mode=="REPLACE":gdk_mode = GDK.PROP_MODE_REPLACE
    elif mode=="APPEND":gdk_mode = GDK.PROP_MODE_APPEND
    elif mode=="PREPEND":gdk_mode = GDK.PROP_MODE_PREPEND
    try:
        window.property_change(atom, int_atom, format, gdk_mode, data)
        return 1
    except: return 0

class MemInfo(gtk.GtkWindow):
    def __init__(self, dock):
        self.__dock = dock
        gtk.GtkWindow.__init__(self)
        self.set_title("Meminfo")
        self.set_name("Meminfo")
        if dock:
            self.__mem_free = gtk.GtkButton()
	    self.__mem_free.connect("clicked", self.OnClicked)
            icon = gtk.GtkPixmap(self, "/usr/X11R6/include/X11/pixmaps/meminfo.xpm")
            self.__mem_free.add(icon)
            self.add(self.__mem_free)
        else:
            self.__mem_free = gtk.GtkProgressBar()
            self.__mem_free.set_show_text(1)
            frame = gtk.GtkAspectFrame("Free Memory", 0.5, 0.5, 0,1)
            frame.add(self.__mem_free)
            self.add(frame)
            self.__mem_free.set_format_string("%p%%")
        self.connect("destroy", self.OnDestroy)
        self.realize()
        if dock:
            set_gdk_win_prop(self.get_window(), "_NET_WM_WINDOW_TYPE",
                             "ATOM", 32, "REPLACE",
                             [gdk_intern_atom("_NET_WM_WINDOW_TYPE_DOCK")])
        self.show_all()
        self.OnTimeout()
        self.__timeout = gtk.timeout_add(4000, self.OnTimeout)
    def OnDestroy(self, widget):
        if self.__timeout:gtk.timeout_remove(self.__timeout)
        gtk.mainquit()
    def OnClicked(self, widget):
	# - Should present a window with full APM status & Quit option
	self.destroy()
    def OnTimeout(self):
        meminfo = open("/proc/meminfo", "r").readlines()
        for line in meminfo:
            parts = line.split()
            if parts[0] == "MemTotal:": mem_total = float(parts[1])
            if parts[0] == "MemFree:": mem_free = float(parts[1])
            if parts[0] == "Cached:": mem_cached = float(parts[1])
            if parts[0] == "SwapFree:": mem_swap_free = float(parts[1])
            if parts[0] == "SwapTotal:": mem_swap = float(parts[1])
        mem_level = (mem_free+mem_cached+mem_swap_free)/(mem_total+mem_swap)
        if self.__dock:
            midlevel=0.5
            style = self.__mem_free.get_style().copy()
            red = 65535*max(0.0, min(1.0, (100.0-mem_level)/(100.0-midlevel)))
            green = 65535*max(0.0, min(1.0, (mem_level/midlevel)))
            if red>32767: red = red-65536
            if green>32767: green = green-65536
            style.bg[gtk.STATE_NORMAL] = self.get_colormap().alloc(red,
                                                                   green,
                                                                   0)
            self.__mem_free.set_style(style)
        else:
            self.__mem_free.update((mem_free+mem_cached)/mem_total)        
        return 1

def Main(argv):
    dock = 0
    if "--dock" in argv: dock=1
    meminfo = MemInfo(dock)
    gtk.mainloop()

if __name__ == "__main__": Main(sys.argv)
